/*
 * $FreeBSD$
 */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <libnet.h>

#define LEET 31337

/*
 * Packet structures and defines.
 */

struct gre_h {
        u_int16_t flags;
        u_int16_t ptype;
};
struct greip {
        struct ip gi_i;
        struct gre_h  gi_g;
};
struct pkt { 
	struct libnet_ip_hdr	ip;
	struct gre_h		gre;
	struct libnet_ip_hdr	ipip;
	struct libnet_icmp_hdr	icmp;
};

#define gi_pr           gi_i.ip_p
#define gi_len          gi_i.ip_len
#define gi_src          gi_i.ip_src
#define gi_tun_dest          gi_i.ip_tun_dest
#define gi_ptype        gi_g.ptype
#define gi_flags        gi_g.flags
#define GRE_CP          0x8000  /* Checksum Present */
#define GRE_RP          0x4000  /* Routing Present */
#define GRE_KP          0x2000  /* Key Present */
#define GRE_SP          0x1000  /* Sequence Present */
#define GRE_SS          0x0800  /* Strict Source Route */

/* 
 * endpoints to generate packet with.
 * XXX: don't be so lazy and make them user-configurable.
 */
static const char tun_src_str[]		= "172.16.1.2";
static const char tun_dest_str[]	= "172.16.1.1";

/*
 * function main().
 */
int
main(int argc, char *argv[])
{
	int s, err, iplen;
	struct pkt	gip;
	struct in_addr	tun_src;
	struct in_addr	phys_dest;
	struct in_addr	tun_dest;
	char		*phys_dest_str;

	if (argc != 2) {
		fprintf(stderr, "usage: greping [dest]\n"
			"dest must be an IP address, not a hostname.\n");
		exit(EXIT_FAILURE);
	}

	/*
	 * no need to look up own address - ip_output()
	 * in the kernel is smart enough to do it for us
	 * and in an interface-independent manner.
	 */

	/*
	 * physical IP target of tunnel
	 */
	phys_dest_str = argv[1];
 	phys_dest.s_addr = inet_addr(phys_dest_str);

	/*
	 * tunnel address endpoints
	 */
 	tun_src.s_addr = inet_addr(tun_src_str);
 	tun_dest.s_addr = inet_addr(tun_dest_str);

	s = libnet_open_raw_sock(IPPROTO_IP);
	if (s == -1) {
		perror("libnet_raw_sock");
		exit(EXIT_FAILURE);
	}
	
	/* build packet from outside in. */
	iplen = sizeof(gip);
	bzero(&gip, iplen);

	libnet_build_ip(iplen - LIBNET_IP_H,
		0, 0, 0, 255, IPPROTO_GRE, INADDR_ANY, phys_dest.s_addr,
		NULL,
		0,
		(void *)&gip.ip);
	gip.gre.flags = 0;
	gip.gre.ptype = htons(ETHERTYPE_IP);
	libnet_build_ip(LIBNET_ICMP_ECHO_H,
		IPTOS_LOWDELAY|IPTOS_THROUGHPUT,
		LEET, 0, 255, IPPROTO_ICMP, tun_src.s_addr, tun_dest.s_addr,
		(void *)&gip.icmp, LIBNET_ICMP_H+LIBNET_ICMP_ECHO_H,
		(void *)&gip.ipip);
	libnet_build_icmp_echo(ICMP_ECHO, 0, LEET, LEET, NULL,
		0, (void *)&gip.icmp);
	/*
	 * checksum the inside of the tunneled packet.
	 * we do not need to compute outer shell's checksum, as
	 * ip_output() does it for us.
	 */
	libnet_do_checksum((void*)&gip.ipip, IPPROTO_ICMP, ICMP_ECHO_H);
	libnet_do_checksum((void*)&gip.ipip, IPPROTO_IP,
		LIBNET_IP_H+LIBNET_ICMP_H+LIBNET_ICMP_ECHO_H);
	
	err = libnet_write_ip(s, (void*) &gip, iplen);
	usleep(50000);

	if (err == -1) perror("libnet_write_ip");
	libnet_close_raw_sock(s);

	exit(EXIT_SUCCESS);
}
