/*
 * $FreeBSD$
 */

#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <err.h>
#include <errno.h>

#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/sockio.h>

#include <netdb.h>
#include <net/if.h>
#include <net/if_var.h>
#include <net/route.h>

#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/in_var.h>
#include <netinet/ip.h>
#include <netinet/ip_var.h>
#include <arpa/inet.h>

/*
 * prototypes
 */

static void usage(void);

/*
 * function main().
 */
int
main(int argc, char *argv[])
{
	int i, s, err, sinflen;
	char			*inf;
	struct ifreq		ifr;
	struct sockaddr_in	sin, sin_f;
	char buf[1024];

	while ((i = getopt(argc, argv, "i:")) != -1)	{
		switch(i) {
		case 'i':
			inf = optarg;
			break;
		default:
			usage();
			exit(EXIT_FAILURE);
		}
	}

	if (strncmp("gre", inf, 3) != 0) {
		usage();
		exit(3);
	}

	s = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if (s == -1) {
		errx(errno, "socket");
	}

	bzero(&ifr, sizeof(ifr));
	strncpy(ifr.ifr_name, inf, sizeof(ifr.ifr_name));
	err = ioctl(s, SIOCGIFADDR, (caddr_t)&ifr);
	if (err < 0) {
		errx(errno, "SIOCGIFADDR");
	}

	bcopy(&ifr.ifr_addr, &sin, sizeof(sin));
	sin.sin_port = htons(6969);
	err = bind(s, (struct sockaddr *) &sin, sizeof(sin));
	if (err < 0) {
		errx(errno, "bind");
	}
	printf("%s: ip is %s\n", inf, inet_ntoa(sin.sin_addr));

	bzero(&sin_f, sizeof(sin_f));
	sinflen = sizeof(sin_f);
	for (;;) {
		i = recvfrom(s, (void *)buf, sizeof(buf),
			     0, (struct sockaddr *) &sin_f, &sinflen);
		if (i == -1) {
			errx(errno, "recvfrom");
		}
		printf("%s: packet size %d received from %s.\n",
		       inf, i, inet_ntoa(sin_f.sin_addr));
	}

	exit(EXIT_SUCCESS);
}

static void
usage(void)
{
	fprintf(stderr, "usage: grerecv -i iface\n");
}
