/*	$FreeBSD$ */

/*
 * Copyright (c) 2001 Organic Systems.
 * All rights reserved
 *
 * This code is conceptually based on the work of the WIDE project
 * and earlier code by Heiko W. Rupp.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *        This product includes software developed by Organic
 *        Systems and its affiliates.
 * 4. Neither the name of Organic Systems nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ORGANIC SYSTEMS AND AFFILIATES
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * if_gre.h
 */

#ifndef _NET_IF_GRE_H_
#define _NET_IF_GRE_H_

/*
 * Per-instance tunnel structure.
 * XXX: these must change to hold IPv6 addresses in future 
 */
struct gre_softc {
	struct ifnet	gre_if;		/* ifnet control structure */
	struct in_addr	gre_src;	/* physical source of GRE packets */
	struct in_addr	gre_dst;	/* physical target of GRE packets */
	u_char		gre_proto;	/* protocol of encapsulator */
	u_char		pad00[3];	/* padding */
	struct route	gre_route;	/* computed route */
	u_long		gre_flags;	/* per-instance flags */
	u_long		gre_key;	/* tunnel key */
	struct resource	*gre_unit;	/* resource allocation */
	TAILQ_ENTRY(gre_softc)	gre_link;	/* for linking */
};

struct gre_h {
	u_int16_t flags;
	u_int16_t ptype;
	/* optional fields:
	 * u_int16_t checksum;
	 * u_int16_t reserved1;
	 */
};

/* maximum size of a GRE header with all options set */
#define GRE_H_MAXLEN	(16)

struct greip {
	struct ip	gi_i;
	struct gre_h	gi_g;
};

/*
 * header fields
 */
#define gi_pr           gi_i.ip_p
#define gi_len          gi_i.ip_len
#define gi_src          gi_i.ip_src
#define gi_dst          gi_i.ip_dst
#define gi_ptype        gi_g.ptype
#define gi_flags        gi_g.flags

/*
 * header flag values
 */
#define GRE_CP		 0x8000	/* Checksum Present */
#define GRE_VER_MASK	 0x0007	/* 3 bits for GRE version */
#define GRE_RFC1701_MASK 0x7FF8	/* Mask for RFC1701 options */

#define GRE_VERSION_0	0	/* RFC 1701 or 2784 GRE */
#define GRE_VERSION_1	1	/* Microsoft PPTP */

#if 0
#define GRE_RP          0x4000  /* Routing Present */
#define GRE_KP          0x2000  /* Key Present */
#define GRE_SP          0x1000  /* Sequence Present */
#define GRE_SS          0x0800  /* Strict Source Route */
#endif

/*
 * defines
 */

/* 
 * default maximum number of tunnels.
 */
#ifndef NGRE
#define NGRE 16
#endif /* NGRE */

#ifndef GRE_NEST_LIMIT
#define GRE_NEST_LIMIT	2	/* limit for nested interfaces */
#endif /* GRE_NEST_LIMIT */

#define GRENAME		"gre"
#define GREDEV		"if_gre"
#define GIE_MAXUNIT     0x7fff  /* ifp->if_unit is only 15 bits */

/*
 * code currently ties its MTU to the underlying physical interface.
 */
#define GRE_DEFAULT_MTU	1476	/* sizeof(eth_h) - greip */
#define GRE_MTU_MIN	552	/* MTU of IP-over-X.25 - greip */
#define GRE_MTU_MAX	16384	/* for *really* fast links */

/*
 * defined ioctls, as per NetBSD/OpenBSD.
 * these are only supported in the interests of portability.
 */
#ifndef GRESADDRS
#define GRESADDRS	_IOW('i', 101, struct ifreq)
#define GRESADDRD	_IOW('i', 102, struct ifreq)
#define GREGADDRS	_IOWR('i', 103, struct ifreq)
#define GREGADDRD	_IOWR('i', 104, struct ifreq)
#define GRESPROTO	_IOW('i', 105, struct ifreq)
#define GREGPROTO	_IOWR('i', 106, struct ifreq)
#endif

/*
 * extern prototypes
 */

/* if_gre.c */
int gre_clone_create __P((struct if_clone *ifc, int *unit));
void gre_clone_destroy __P((struct ifnet *ifp));
int gre_ioctl __P((struct ifnet *ifp, u_long cmd, caddr_t data));
void gre_input __P((register struct mbuf *m, int off, int proto));
struct gre_softc *gre_lookup __P((struct mbuf *m, u_int8_t proto));
int gre_output __P((struct ifnet *ifp, struct mbuf *m, struct sockaddr *dst, struct rtentry *rt));
#endif /* _NET_IF_GRE_H_ */
